#!/usr/bin/perl -w

###
# Project:     pflogstats
# Module:      pflogstats-common-support.pm
# Description: Support functions
# Copyright:   Dr. Peter Bieringer <pbieringer at aerasec dot de>
#               AERAsec GmbH <http://www.aerasec.de/> 
# License:     GNU GPL v2
# CVS:         $Id: pflogstats-common-support.pm,v 1.21 2003/07/01 07:59:51 rootadm Exp $
###

###
# ChangeLog:
#	0.01
#	 - initial creation
#	0.02
#	 - add option "numberformat"
#	0.03
#	 - minor code movement
#	0.04
#	 - add statistics
#	0.05
#	 - move statistics to separate module
#	0.06
#	 - move networking code to separate module
#	0.07
#	 - check debug value string for numeric value
#	0.08
#	 - hash reference cosmetics
#	0.09
#	 - replace all hash references with proper code
#       0.10
#        - make Perl 5.0 compatible
#	0.11
#	 - add new switch "verbose"
###

## Todo:
# - correct module function export
##

use strict;

use Sys::Hostname;
use Number::Format;
use Time::Local;
use POSIX qw(strftime);
use locale;


#package pflogstats::common::support;
#package pflogstats_common_support;


## Local constants
my $module_type = "common";
my $module_name = $module_type . "-support";
my $module_version = "0.10";

## Export module info
$main::moduleinfo{$module_name}->{'version'} = $module_version;
$main::moduleinfo{$module_name}->{'type'} = $module_type;
$main::moduleinfo{$module_name}->{'name'} = $module_name;

## Global prototyping
#sub print_timerange_normal();


## Local prototyping

## Global variables

# Number format
my $numberformat_de_DE = new Number::Format(-thousands_sep   => '.', -decimal_point   => ',', -int_curr_symbol => 'EUR');
my $numberformat_en_US = new Number::Format(-thousands_sep   => ',', -decimal_point   => '.', -int_curr_symbol => '$');
$main::numberformat{'de_DE'} = $numberformat_de_DE;
$main::numberformat{'en_US'} = $numberformat_en_US;


## Register options
$main::options{'mydomainname=s'} = \$main::opts{'mydomainname'};
$main::options{"numberformat=s"}  = \$main::opts{'numberformat'};
$main::options{"debug|d=s"}  = \$main::opts{'debug'};
$main::options{'format=s'} = \@main::opt_format;
$main::options{'verbose'} = \$main::opts{'verbose'};


## Register calling hooks
$main::hooks{'beforemainloopstarts'}->{$module_name} = \&beforemainloopstarts1;
$main::hooks{'help'}->{$module_name} = \&help1;
$main::hooks{'checkoptions'}->{$module_name} = \&checkoptions;


## Local variables
my @net_excluded;


## Global callable functions
sub help1() {
	my $temp;
	my $helpstring = "
    General:
    [--mydomainname <domain>]    My domain (default: domain of hostname)
";

	# Number format
	$temp = "    [--numberformat <format]  Number format\n                               Default: 'de_DE'\n                               Available support:";
	foreach my $format (keys %main::numberformat ) {
	        $temp .= " '" . $format . "'";
	};
	$helpstring .= $temp . "\n";

	# Debug value
	$temp = "    [--debug|-d <debugvalue>]  Display some debug information";
	$helpstring .= $temp . "\n";

	# Verbose option
	$temp = "    [--verbose]  Be more verbose";
	$helpstring .= $temp . "\n";

	# Format options
	$temp = "    [--format <formatvalue>]  Format option for displaying information (can be used more than once)\n             Default: 'txttable'\n                         Available formats: 'txttable' 'computer' 'treeview' 'indented'\n               (not all formats are supported on each type)";
	$helpstring .= $temp . "\n";


	return $helpstring;
};

# Check options
sub checkoptions() {

	##  Check debug value
	if (! defined $main::opts{'debug'}) {
		$main::opts{'debug'} = 0;
	};

	# hex to dec
	if ( $main::opts{'debug'} =~ /^0x[0-9A-Fa-f]+$/i ) {
		$main::opts{'debug'} = hex ($main::opts{'debug'});
	};

	# check dec
	if ( ! ( $main::opts{'debug'} =~ /^[0-9]+$/i ) ) {
		die "Debug value not decimal: " . $main::opts{'debug'} . "\n";
	};

	if ( $main::opts{'debug'} != 0 ) {
		printf STDERR "DEBUG: Debug mode: %x\n", $main::opts{'debug'};
	};


	## Set domainname
	if (! defined $main::opts{'mydomainname'} ) {
		# Set default
		my $myhostname = hostname;
		if ( $myhostname =~ /^[^\.]+\.(\S+)/ ) {
			$main::opts{'mydomainname'} = $1;
		};
	};
	print STDERR "INFO: Domainname: " . $main::opts{'mydomainname'} .  "\n" if ($main::opts{'debug'} & 0x0040);

	## Set hostname
	if (! defined $main::opts{'myhostname'} ) {
		# Set default
		$main::opts{'myhostname'} = hostname;
	};
	print STDERR "INFO: Hostname: " . $main::opts{'myhostname'} .  "\n" if ($main::opts{'debug'} & 0x0040);


	##  Check number format
	if (! defined $main::opts{'numberformat'}) {
		# Default
		$main::opts{'numberformat'} = "de_DE";
	};
	if (! defined $main::numberformat{$main::opts{'numberformat'}}) {
		die "ERROR: Unsupported number format: " . $main::opts{'numberformat'} . "\n";
	};


	## Check format options
	if (scalar (@main::opt_format) == 0) {
		# Default
		push @main::opt_format, 'txttable';
	};
	for my $key (@main::opt_format) {
		# check key
		if ($key ne "computer" && $key ne "treeview" && $key ne "indented" && $key ne "txttable") {
			die "ERROR: Unsupported format: " . $key . "\n";
		} else {
			$main::format{$key} = 1;
		};
	};
};

sub beforemainloopstarts1() {
};


# Unixtime2string
sub unixtime2string($) {
	return POSIX::strftime "%a %b %e %H:%M:%S %Y", localtime($_[0]);
};

# Print time range
sub print_timerange_normal() {
	my $timeminstr = POSIX::strftime "%a %b %e %H:%M:%S %Y", localtime($main::timemin);
	my $timemaxstr = POSIX::strftime "%a %b %e %H:%M:%S %Y", localtime($main::timemax);
	printf " Begin: %s (localtime)\n", $timeminstr;
	printf " End  : %s (localtime)\n", $timemaxstr;
};


# Print statistics
sub print_stat($$) {
	my $title = shift;
	my $p_stat = shift;
	my %stat = %$p_stat;
	my $sum = 0;

	print '='x75 . "\n";
	print "$title\n";
	print_timerange_normal();
	print '-'x75 . "\n";
	if ($main::opts{'sort_type'} eq "alpha") {
		for my $key (sort keys %stat) {
			printf "%-67s : %5d\n", $key, $stat{$key};
			$sum += $stat{$key};
		};
	} elsif ($main::opts{'sort_type'} eq "maxmin") {
		for my $key (sort { $stat{$b} <=> $stat{$a} } keys %stat) {
			printf "%-67s : %5d\n", $key, $stat{$key};
			$sum += $stat{$key};
		};
	} elsif ($main::opts{'sort_type'} eq "minmax") {
		for my $key (sort { $stat{$a} <=> $stat{$b} } keys %stat) {
			printf "%-67s : %5d\n", $key, $stat{$key};
			$sum += $stat{$key};
		};
	} else {
		die "ERROR(missing-code): Unsupported sort type";
	};
	print '-'x75 . "\n";
	printf "%-67s : %5d\n", "Total", $sum;
	print '='x75 . "\n\n";
};


# Print time range
sub print_timerange($) {
	my $format = shift || die "Missing format (arg1)";

	if (! defined $main::timemin || ! defined $main::timemax) {
		warn "No timerange data!\n";
		return;
	};

	if ( $format eq "computer" ) {
		printf "__timemin=%d\n", $main::timemin;
		printf "__timemax=%d\n", $main::timemax;
	} else {
		print '+'x75 . "\n";
		if (defined $main::timemin && defined $main::timemax) {
			my $timeminstr = POSIX::strftime "%a %b %e %H:%M:%S %Y", localtime($main::timemin); 
			my $timemaxstr = POSIX::strftime "%a %b %e %H:%M:%S %Y", localtime($main::timemax); 
			printf "Timerange:  %s  -  %s\n", $timeminstr, $timemaxstr;
		} else {
			printf "Timerange:  NO DATA!\n";
		};
		print '+'x75 . "\n";
		printf "\n";
	};
};


# Print headline
sub print_headline($$) {
	my $text = shift || die "Missing text (arg1)";
	my $format = shift || die "Missing format (arg2)";

	if ( $format eq "computer" ) {
		printf "\n";
		printf "__headline=\"%s\"\n", $text;
	} else {
		printf "\n\n";
		print '#'x75 . "\n";
		printf "%s\n", $text;
		print '#'x75 . "\n";
		printf "\n";
	};
};


## Print a subtreeview
sub print_subtreeview($) {
	my $p0 = $_[0] || die "Missing hash pointer";

	# if (! defined $p) { die "hash pointer is undefined"; };
	# if (! defined %$p) { die "hash of pointer is undefined"; };

	my $c0 = scalar( keys %$p0 );

	foreach my $k0 ( sort keys %$p0 ) { 
		$c0--;
		if ( $c0 > 0 ) {
			print "+- ";
		} else {
			print "`- ";
		};
		my $p1 = $$p0{$k0};
		my $c1 = scalar( keys %$p1 );
		#print $k0 . " [" . $c1 . "]\n";
		print $k0 . "\n";
		foreach my $k1 ( sort keys %$p1 ) {
			$c1--;
			if ( $c0 > 0 ) {
				print "|";
			} else {
				print " ";
			};
			my $p2 = $$p1{$k1};
			if ( $c1 > 0 ) {
				print "  +- ";
			} else {
				print "  `- ";
			};
			my $c2 = scalar( keys %$p2 );
			#print $k1 . " [" . $c2 . "]\n";
			print $k1 . "\n";
			foreach my $k2 ( sort keys %$p2 ) {
				$c2--;
				if ( $c0 > 0 ) {
					print "|";
				} else {
					print " ";
				};
				if ( $c1 > 0 ) {
					print "  |";
				} else {
					print "   ";
				};
				if ( $c2 > 0 ) {
					print "  +- ";
				} else {
					print "  `- ";
				};
				print $k2 . " [" . $$p2{$k2} . "]\n";
			};
		};
	};
};

## Print a treeview
# $1: pointer to treehash
# $2 (optional): list of keys
sub print_treeview($;@) {
	my $p = $_[0] || die "Missing hash pointer";
	my $list = $_[1]; # optional list of keys
	my $p0;

	#print "List: " . $list . "\n";
	# print "DEBUG/main/print_treeview: called\n";
	
	if (! defined $p) { die "hash pointer is undefined"; };
	if (! defined %$p) { die "hash of pointer is undefined"; };
	if (! defined $list) { $list = "" };

	if ($list eq "" ) {
		# print "DEBUG/main/print_treeview: without list\n";
		for my $k ( sort keys %$p ) {
			# print "DEBUG/main/print_treeview: " . $k . "\n";
			$p0 = $$p{$k};
			#print "\n" . $k . " [" . scalar(keys %$p0)  . "]\n";
			print "\n" . $k . "\n";
			print_subtreeview( \%$p0 );
		};
	} else {
		foreach my $k (split " ", $list) {
			print "\n" . $k . "\n";
			if (! defined $$p{$k} ) {
				print " !  no data !\n";
				next;
			};
			#print "\n" . $k . " [" . scalar(keys %$p0)  . "]\n";
			print "\n" . $k . "\n";
			$p0 = $$p{$k};
			print_subtreeview( \%$p0 );
		};
	};
};

## Get Domain
sub extract_domain ($) {
	my $email = shift || return "";
	#die "Missing argument";
	my $domain;

	if ( $email eq "from=<>" || $email eq "from=<#@[]>" ) {
		# unspecified sender, nothing to strip
		return $email;
	};
	
	if ( $email =~ /^.*@([^@]+)$/ ) {
		$domain = $1;
	} else {
		# email address contains no domain
		$domain = "ADDRESS_WITHOUT_DOMAIN";
	};

	if (! defined $domain ) {
		return "ADDRESS_WITH_INVALID_DOMAIN";
	};

	return $domain;
};


## Get 2nd Level Domain
sub extract_2ndleveldomain ($) {
	my $email = shift || die "Missing argument";
	my $domain;

	if ( $email eq "from=<>" || $email eq "from=<#@[]>" ) {
		# unspecified sender, nothing to strip
		return $email;
	};

	# Check for e-mail address
	if ( $email =~ /@/ ) {
		# Is a e-mail address	
		if ( $email =~ /^.*@([^@]+)$/ ) {
			$domain = $1;
		} else {
			$domain = "ADDRESS_WITHOUT_DOMAIN";
		};

		if (! defined $domain ) {
			return "ADDRESS_WITH_INVALID_DOMAIN";
		};
	} else {
		# domain only
		$domain = $email;
	};

	# Strip trailing "."
	$domain =~ s/\.+$//g;

	if ( $domain =~ /^[^.]+\.[^.]+$/ ) {
		return $domain;
	} elsif ( $domain =~ /^.*\.([^.]+\.[^.]+)$/ ) {
		$domain = $1;
		if (! defined $domain ) {
			$domain = "ADDRESS_WITHOUT_SECONDLEVEL_DOMAIN";
		};
		return $domain;
	} else {
		return "ADDRESS_WITHOUT_SECONDLEVEL_DOMAIN";
	};
};


## End of module
return 1;
