#!/usr/bin/perl -w

###
# Project:     pflogstats
# Module:      pflogstats-statistics-reject.pm
# Type:        statistics
# Description: Statistics module for accounting
# Copyright:   Dr. Peter Bieringer <pbieringer at aerasec dot de>
#               AERAsec GmbH <http://www.aerasec.de/> 
# License:     GNU GPL v2
# CVS:         $Id: pflogstats-statistics-reject.pm,v 1.11 2003/05/22 14:02:23 rootadm Exp $
###

## ChangeLog
#	0.01
#	 - initial split-off
#	0.02
#	 - add hook "rblblocked"
#	0.03
#	 - print timerange on table header, remove "Counts" on second column header
#	 - reenable domain treeview code
#	0.04
#	 - support new format code style
#	0.05
#	 - adjust loglineparser arguments
#	0.06
#	 - reimplement logline parser for standalone usage (no longer old pflogsumm code is used)
#	0.07
#	 - fix a bug in qid match (to catch older postfix versions loglines, too)
#	0.08
#	 - enhance bugfix in qid match
#       0.09
#        - make Perl 5.0 compatible
##

use strict;


## Local constants
my $module_type = "statistics";
my $module_name = $module_type . "-reject";
my $module_version = "0.09";

package pflogstats::statistics::reject;

## Export module info
$main::moduleinfo{$module_name}->{'version'} = $module_version;
$main::moduleinfo{$module_name}->{'type'} = $module_type;
$main::moduleinfo{$module_name}->{'name'} = $module_name;
# Global prototyping


## Local prototyping


## Register options


## Register calling hooks
$main::hooks{'printstatistics'}->{$module_name} = \&printstatistics;
$main::hooks{'checkoptions'}->{$module_name} = \&checkoptions;
$main::hooks{'help'}->{$module_name} = \&help;
$main::hooks{'loglineparser'}->{$module_name} = \&loglineparser;

# Define type
$main::types{'reject'} = 0;

## Global variables

## Local variables
my %rejectTreeview;
my %rejects;

## Global callable functions

# Help
sub help() {
	my $helpstring = "
    Type: reject
    [--treeview [domain]]        Display treeview of statistics
    [--debug <debug>]            Debug value
                                  | 0x0001 : show reject reason log part
";
	return $helpstring;
};


# Check options
sub checkoptions() {
};


# Parse logline
sub loglineparser(\$\$) {
	return if ( $main::types{'reject'} == 0);

	if (! defined $_[0]) { die "Missing time pointer (arg1)"; };
	if (! defined $_[1]) { die "Missing logline pointer (arg2)"; };

	# Currently nothing to do here
	
	my ($rejectinfo);
	my ($from, $to, $from_domain, $to_domain, $rblserver, $relay);

	my ($rejFrom, $rejTyp, $rejRmdr, $rejData, $rejReas);

	# Example: Jan  1 02:55:16 host postfix/smtpd[13756]: D00C713866: reject: ....
	#print "DEBUG(uce/loglineparser): get: '" . ${$_[1]} . "\n";
	if (! ( ${$_[1]} =~ /^.*\spostfix\/smtpd\[[0-9]+\]: ([A-F0-9]{8,11}: )?reject: (.*)$/o ) ) {
		# Unmatched logline
		return;
	};

	$rejectinfo = $2;

	# First: get everything following the "reject: " token
	($rejTyp, $rejFrom, $rejRmdr) = $rejectinfo =~ /^([^ ]+) from (\S+): (.*)$/o;

	# Next: get the reject "reason"
	$rejReas = $rejRmdr;

	if ($rejTyp eq "RCPT") {
		$rejReas =~ s/^(?:.*?[:;] )(?:\[[^\]]+\] )?([^;,]+)[;,].*$/$1/oi;
	} else {
		$rejReas =~ s/^(?:.*[:;] )?([^,]+).*$/$1/o;
	};

	($from, $to) = $rejRmdr =~ / from=<([^>]*)> to=<([^>]*)>/o;
	if ($from eq "") { $from = "from=<>" };
	if ($from eq "#@[]") { $from = "from=<#@[]>"; };
	$from = lc ($from);
	$to   = lc ($to);

	# Currently not needed
	# Hook "modifyaddress"
	for my $p_hook (keys %{$main::hooks{'modifyaddress'}}) {
		$from = &{$main::hooks{'modifyaddress'}->{$p_hook}} ($from);
		$to   = &{$main::hooks{'modifyaddress'}->{$p_hook}} ($to);
	};

	$to_domain = ::extract_domain($to);
	$from_domain = ::extract_domain($from);

	#print "DEBUG(acc/loglineparser): rejReas: '" . $rejReas . "\n";

	if($rejReas =~ m/^Sender address rejected:/o) {
		# Sender address rejected: Domain not found
		# Sender address rejected: need fully-qualified address
		# Sender address rejected: undeliverable sender address: ...
		if ($rejReas =~ /undeliverable sender address:/o) {
			$rejReas = "Sender address rejected: undeliverable sender address" ;
		} elsif ($rejReas =~ /unverified sender address:/o) {
			$rejReas = "Sender address rejected: unverified sender address" ;
		}

		# Fill reject data

	} elsif($rejReas =~ m/^Recipient address rejected:/o) {
		# Recipient address rejected: Domain not found
		# Recipient address rejected: need fully-qualified address

		# Fill reject data

	} elsif($rejReas =~ s/^.*?\d{3} (Improper use of SMTP command pipelining);.*$/$1/o) {
		# Fill reject data

	} elsif($rejReas =~ s/^.*?\d{3} (Message size exceeds fixed limit);.*$/$1/o) {
		# Fill reject data

	} elsif ( $rejReas =~ /blocked using ([^ ]+)$/o ) {
		# Example: Jan  1 02:55:16 host postfix/smtpd[13756]: D00C713866: reject: RCPT from unknown[212.251.10.251]: 554 Service unavailable; Client host [212.251.10.251] blocked using relays.ordb.org; This mail was handled by an open relay - please visit <http://ORDB.org/lookup/?host=212.251.10.251>; from=<bulkemail@flash.net> to=<account@example.com> proto=ESMTP helo=<jcsc.nato.int>

		#print "DEBUG(reject/loglineparser/RBL): get: '" . ${$_[1]} . "\n";

		# Fill reject data
		$rejReas = 'Client host rejected: by RBL';

	} elsif ( $rejRmdr =~ /Client host rejected: cannot find your hostname/oi ) {
		# Example: Feb  1 15:29:46 host postfix/smtpd[10407]: E20D81386D: reject: RCPT from unknown[68.112.217.231]: 450 Client host rejected: cannot find your hostname, [68.112.217.231]; from=<martina4048c@lycos.com> to=<account@example.com> proto=SMTP helo=<mail.lycos.com>
		# Fill reject data
		$rejReas = 'Client host rejected: cannot find your hostname';

	} else {
		# Unhandled reject data
		#$rejData = ::extract_domain($rejFrom);
		#$rejData = $rejFrom;
	};

	$rejData = $from . " (" . $rejFrom . ")";

	# Fill data into hash
	$rejectTreeview{$to_domain}->{$to}->{$rejReas}->{$rejData}++;
	++$rejects{$rejTyp}{$rejReas}{$rejData};
};


# print statistics
sub printstatistics() {
	return if ( $main::types{'reject'} == 0);

	my $counter;
	my $format;

	# Format: treeview
	if (defined $main::format{"treeview"}) {
		$format = "treeview";
		my $info = "";
		$info .= " (treeview)";
		$main::opts{'show_treeview'} =~ s/,/ /g;
		if ($main::opts{'show_treeview'} ne "") {
			$info .= ":\n " . $main::opts{'show_treeview'};	
		};
		::print_headline("REJECT statistics " . $info, $format);
		::print_timerange($format);
		::print_treeview( \%rejectTreeview, $main::opts{'show_treeview'} );
	};

	# Format: computer
	if (defined $main::format{"computer"}) {
		$format = "computer";
		print "\n\nWARNING(reject): Format '" . $format . "' is currently not supported!\n\n";
	};

	# Format: indented
	if (defined $main::format{"indented"}) {
		$format = "indented";
		print "\n\nWARNING(reject): Format '" . $format . "' is currently not supported!\n\n";
	};

	# Format: txttable
	if (defined $main::format{"txttable"}) {
		$format = "txttable";
		::print_headline("REJECT statistics", $format);
		::print_timerange($format);

		print '='x75 . "\n";
		printf "Reason\n";
		::print_timerange_normal();
		print '-'x75 . "\n";
		my $mastercounter = 0;
		for my $rejTyp (sort keys %rejects) {
			for my $rejReas (sort keys %{$rejects{$rejTyp}}) {
				my $counter = 0;
				for my $rejRmdr (keys %{$rejects{$rejTyp}->{$rejReas}}) {
					$counter += $rejects{$rejTyp}->{$rejReas}->{$rejRmdr};
				};
				printf "%-65s : %7d\n", substr($rejTyp . ":" . $rejReas,0, 65), $counter;
				$mastercounter += $counter;
			};
		};		
		print '-'x75 . "\n";
		printf "%-65s : %7d\n", "Sum:", $mastercounter;
		print '='x75 . "\n";
	};
};



## Local functions




## End of module
return 1;
